VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "dwRegion"
Attribute VB_Creatable = True
Attribute VB_Exposed = True
Option Explicit

' Class dwRegion
' Region object control and configuration class
' Copyright (c) 1996 by Desaware Inc.
' Part of the Desaware API Classes Library

' A number of functions require a group of data in a single
' parameter.  This task is traditionally handled by an array.
' Visual Basic 4.0 has a new data type called a Collection.
' This class library uses Collections, but if you prefer
' arrays, or your program uses arrays extensivly, then setting
' this flag to True will switch those functions to their
' array-using equivalents.
#Const FlagUseArrayNotCollection = False


#If Win32 Then
Private hInternalRgn As Long
#Else
Private hInternalRgn As Integer
#End If

#If Win32 Then
Private Declare Function apiCombineRgn& Lib "gdi32" Alias "CombineRgn" (ByVal hDestRgn As Long, ByVal hSrcRgn1 As Long, ByVal hSrcRgn2 As Long, ByVal nCombineMode As Long)
Private Declare Function apiCreateEllipticRgn& Lib "gdi32" Alias "CreateEllipticRgn" (ByVal x1 As Long, ByVal y1 As Long, ByVal x2 As Long, ByVal y2 As Long)
Private Declare Function apiCreateEllipticRgnIndirect& Lib "gdi32" Alias "CreateEllipticRgnIndirect" (lpRect As RECT)
Private Declare Function apiCreatePolygonRgn& Lib "gdi32" Alias "CreatePolygonRgn" (lpPoint As POINTAPI, ByVal nCount As Long, ByVal nPolyFillMode As Long)
Private Declare Function apiCreateRectRgn& Lib "gdi32" Alias "CreateRectRgn" (ByVal x1 As Long, ByVal y1 As Long, ByVal x2 As Long, ByVal y2 As Long)
Private Declare Function apiCreateRectRgnIndirect& Lib "gdi32" Alias "CreateRectRgnIndirect" (lpRect As RECT)
Private Declare Function apiCreateRoundRectRgn& Lib "gdi32" Alias "CreateRoundRectRgn" (ByVal x1 As Long, ByVal y1 As Long, ByVal x2 As Long, ByVal y2 As Long, ByVal X3 As Long, ByVal Y3 As Long)
Private Declare Function apiEqualRgn& Lib "gdi32" Alias "EqualRgn" (ByVal hSrcRgn1 As Long, ByVal hSrcRgn2 As Long)
Private Declare Function apiOffsetRgn& Lib "gdi32" Alias "OffsetRgn" (ByVal hRgn As Long, ByVal x As Long, ByVal y As Long)
Private Declare Function apiPtInRegion& Lib "gdi32" Alias "PtInRegion" (ByVal hRgn As Long, ByVal x As Long, ByVal y As Long)
Private Declare Function apiRectInRegion& Lib "gdi32" Alias "RectInRegion" (ByVal hRgn As Long, lpRect As RECT)
Private Declare Function apiSetRectRgn& Lib "gdi32" Alias "SetRectRgn" (ByVal hRgn As Long, ByVal x1 As Long, ByVal y1 As Long, ByVal x2 As Long, ByVal y2 As Long)
Private Declare Function apiPathToRegion& Lib "gdi32" Alias "PathToRegion" (ByVal hDC As Long)

#Else
Private Declare Function apiCombineRgn% Lib "gdi" Alias "CombineRgn" (ByVal hDestRgn As Integer, ByVal hSrcRgn1 As Integer, ByVal hSrcRgn2 As Integer, ByVal nCombineMode As Integer)
Private Declare Function apiCreateEllipticRgn% Lib "gdi" Alias "CreateEllipticRgn" (ByVal x1 As Integer, ByVal y1 As Integer, ByVal x2 As Integer, ByVal y2 As Integer)
Private Declare Function apiCreateEllipticRgnIndirect% Lib "gdi" Alias "CreateEllipticRgnIndirect" (lpRect As RECT)
Private Declare Function apiCreatePolygonRgn% Lib "gdi" Alias "CreatePolygonRgn" (lpPoints As POINTAPI, ByVal nCount As Integer, ByVal nPolyFillMode As Integer)
Private Declare Function apiCreateRectRgn% Lib "gdi" Alias "CreateRectRgn" (ByVal x1 As Integer, ByVal y1 As Integer, ByVal x2 As Integer, ByVal y2 As Integer)
Private Declare Function apiCreateRectRgnIndirect% Lib "gdi" Alias "CreateRectRgnIndirect" (lpRect As RECT)
Private Declare Function apiCreateRoundRectRgn% Lib "gdi" Alias "CreateRoundRectRgn" (ByVal x1 As Integer, ByVal y1 As Integer, ByVal x2 As Integer, ByVal y2 As Integer, ByVal X3 As Integer, ByVal Y3 As Integer)
Private Declare Function apiEqualRgn% Lib "gdi" Alias "EqualRgn" (ByVal hSrcRgn1 As Integer, ByVal hSrcRgn2 As Integer)
Private Declare Function apiOffsetRgn% Lib "gdi" Alias "OffsetRgn" (ByVal hRgn As Integer, ByVal x As Integer, ByVal y As Integer)
Private Declare Function apiPtInRegion% Lib "gdi" Alias "PtInRegion" (ByVal hRgn As Integer, ByVal x As Integer, ByVal y As Integer)
Private Declare Function apiRectInRegion% Lib "gdi" Alias "RectInRegion" (ByVal hRgn As Integer, lpRect As RECT)
Private Declare Sub apiSetRectRgn Lib "gdi" Alias "SetRectRgn" (ByVal hRgn As Integer, ByVal x1 As Integer, ByVal y1 As Integer, ByVal x2 As Integer, ByVal y2 As Integer)
'Function apiPathToRegion is not available in the WIN16 API.
#End If 'WIN32

Private Sub RaiseRgnError(Optional errval)
    Dim useerr%
    If IsMissing(errval) Then
        RaiseRgnError DWERR_APIRESULT
    Else
        RaiseError errval, "dwRegion"
    End If
End Sub

Public Sub CombineRgn(rRegion1 As dwRegion, rRegion2 As dwRegion, ByVal nCombineMode)
Attribute CombineRgn.VB_HelpID = 2488
Attribute CombineRgn.VB_Description = "Help available in Visual Basic Programmer's Guide to Win32 API CD ROM"
    Dim dl&
    
    dl& = apiCombineRgn(hInternalRgn, rRegion1.hRegion, rRegion2.hRegion, nCombineMode)
    If dl& = 0 Then RaiseRgnError
End Sub

Public Function EqualToRgn(rOtherRegion As dwRegion) As Boolean
Attribute EqualToRgn.VB_HelpID = 2524
Attribute EqualToRgn.VB_Description = "Help available in Visual Basic Programmer's Guide to Win32 API CD ROM"
    Dim dl&
    
    dl& = apiEqualRgn(hInternalRgn, rOtherRegion.hRegion)

    If dl& = 0 Then
        EqualToRgn = False
    Else
        EqualToRgn = True
    End If
End Function

Public Sub OffsetRgn(ByVal x As Long, ByVal y As Long)
Attribute OffsetRgn.VB_HelpID = 2589
Attribute OffsetRgn.VB_Description = "Help available in Visual Basic Programmer's Guide to Win32 API CD ROM"
    Dim dl&
    
    dl& = apiOffsetRgn(hInternalRgn, x, y)
    If dl& = 0 Then RaiseRgnError
End Sub

Public Function PointInRegion(pPoint As dwPoint) As Boolean
Attribute PointInRegion.VB_HelpID = 2595
Attribute PointInRegion.VB_Description = "Help available in Visual Basic Programmer's Guide to Win32 API CD ROM"
    Dim dl&
    
    dl& = apiPtInRegion(hInternalRgn, pPoint.x, pPoint.y)
    
    If dl& = 0 Then
        PointInRegion = False
    Else
        PointInRegion = True
    End If
End Function

Public Function RectInRegion(rRect As dwRECT) As Boolean
Attribute RectInRegion.VB_HelpID = 2598
Attribute RectInRegion.VB_Description = "Help available in Visual Basic Programmer's Guide to Win32 API CD ROM"
    Dim dl&
    Dim tempRect As RECT
    
    rRect.CopyToRECT agGetAddressForObject(tempRect.left)
    dl& = apiRectInRegion(hInternalRgn, tempRect)
    
    If dl& = 0 Then
        RectInRegion = False
    Else
        RectInRegion = True
    End If
End Function

Public Sub SetRectRgn(rRect As dwRECT)
Attribute SetRectRgn.VB_HelpID = 2627
Attribute SetRectRgn.VB_Description = "Help available in Visual Basic Programmer's Guide to Win32 API CD ROM"
#If Win32 Then
    Dim dl&
    
    dl& = apiSetRectRgn(hInternalRgn, rRect.left, rRect.top, rRect.right, rRect.bottom)
    If dl& = 0 Then RaiseRgnError
#Else
    apiSetRectRgn hInternalRgn, rRect.left, rRect.top, rRect.right, rRect.bottom
#End If
End Sub

Public Sub CreateRectRgn(Rectangle As dwRECT)
Attribute CreateRectRgn.VB_HelpID = 2512
Attribute CreateRectRgn.VB_Description = "Help available in Visual Basic Programmer's Guide to Win32 API CD ROM"
    Dim tempRect As RECT
    Rectangle.CopyToRECT agGetAddressForObject(tempRect.left)
    
    hInternalRgn = apiCreateRectRgnIndirect(tempRect)

    If hInternalRgn = 0 Then RaiseRgnError
End Sub

Public Sub CreateEllipticRgn(Rectangle As dwRECT)
    Dim tempRect As RECT
    Rectangle.CopyToRECT agGetAddressForObject(tempRect.left)
    
    hInternalRgn = apiCreateEllipticRgnIndirect(tempRect)

    If hInternalRgn = 0 Then RaiseRgnError
End Sub

#If FlagUseArrayNotCollection Then
Public Sub CreatePolygonRgn(PointArray() As dwPoint, ByVal count As Long, ByVal FillMode As Long)
Attribute CreatePolygonRgn.VB_HelpID = 2514
Attribute CreatePolygonRgn.VB_Description = "Help available in Visual Basic Programmer's Guide to Win32 API CD ROM"
    Dim tempPoints() As POINTAPI
    Dim i As Integer
    
    ReDim tempPoints(count - 1)
    For i = 0 To count - 1
        tempPoints(i).x = PointArray(i).x
        tempPoints(i).y = PointArray(i).y
    Next i
    hInternalRgn = apiCreatePolygonRgn(tempPoints(0), count, FillMode)
    If hInternalRgn = 0 Then RaiseRgnError
End Sub
#Else ' use collection version
' colPoints should be a collection of dwPoints
Public Sub CreatePolygonRgn(colPoints As Collection, ByVal FillMode As Long)
    Dim tempPoints() As POINTAPI
    Dim i As Integer
    
    ReDim tempPoints(colPoints.count - 1)
    For i = 1 To colPoints.count
        tempPoints(i - 1).x = colPoints(i).x
        tempPoints(i - 1).y = colPoints(i).y
    Next i
    hInternalRgn = apiCreatePolygonRgn(tempPoints(0), colPoints.count, FillMode)
    If hInternalRgn = 0 Then RaiseRgnError
End Sub
#End If

Public Sub CreateRoundRectRgn(Rectangle As dwRECT, ByVal EllipseWidth As Long, ByVal EllipseHeight As Long)
Attribute CreateRoundRectRgn.VB_HelpID = 2516
Attribute CreateRoundRectRgn.VB_Description = "Help available in Visual Basic Programmer's Guide to Win32 API CD ROM"
    hInternalRgn = apiCreateRoundRectRgn(Rectangle.left, Rectangle.top, Rectangle.right, Rectangle.bottom, EllipseWidth, EllipseHeight)
    If hInternalRgn = 0 Then RaiseRgnError
End Sub

Public Property Get hRegion() As Long
    hRegion = hInternalRgn
End Property

Public Property Let hRegion(vNewValue As Long)
    hInternalRgn = vNewValue
End Property
